#include "nodes/track/trackNode.hpp"

namespace GNode
{

const float IOU_THRESHOLD = 0.7; // 可调阈值

float calculate_iou(const std::vector<float>& tlwh, const data::Box& box) {
    float track_x1 = tlwh[0];
    float track_y1 = tlwh[1];
    float track_x2 = tlwh[0] + tlwh[2];
    float track_y2 = tlwh[1] + tlwh[3];

    float box_x1 = box.left;
    float box_y1 = box.top;
    float box_x2 = box.right;
    float box_y2 = box.bottom;

    float inter_x1 = std::max(track_x1, box_x1);
    float inter_y1 = std::max(track_y1, box_y1);
    float inter_x2 = std::min(track_x2, box_x2);
    float inter_y2 = std::min(track_y2, box_y2);

    float inter_area = std::max(0.0f, inter_x2 - inter_x1) * std::max(0.0f, inter_y2 - inter_y1);

    float track_area = tlwh[2] * tlwh[3];
    float box_area = (box.right - box.left) * (box.bottom - box.top);

    float union_area = track_area + box_area - inter_area;

    if (union_area <= 0.0f) {
        return 0.0f;
    }
    return inter_area / union_area;
}


void TrackNode::work()
{
    printf("TrackNode %s\n", name_.c_str());
    while (running_)
    {
        bool has_data = false;
        for (auto& input_buffer : input_buffers_)
        {
            std::shared_ptr<meta::MetaData> metaData;
            if (!input_buffer.second->try_pop(metaData))
            {
                continue;
            }
            has_data = true;
            // printf("Node %s get data from %s\n", name_.c_str(), input_buffer.first.c_str());

            // auto res = model_->forward(tensor::cvimg(image), image.cols, image.rows, 0.0f, 0.0f);
            if (!tracker_)
            {
                printf("track is nullptr\n");
                continue;
            }
            std::vector<Object> objects;
            std::transform(metaData->boxes.begin(), metaData->boxes.end(), std::back_inserter(objects), [this](data::Box& box) {
                if (box.label == track_label_) {
                    Object obj;
                    obj.rect.x = box.left;
                    obj.rect.y = box.top;
                    obj.rect.width = box.right - box.left;
                    obj.rect.height = box.bottom - box.top;
                    obj.label = box.class_id;
                    obj.prob = box.score;
                    return obj;
                }
            });
            std::vector<STrack> output_stracks = tracker_->update(objects);
            std::vector<bool> box_matched(metaData->boxes.size(), false); // 标记 box 是否已被匹配

            for (const auto& track : output_stracks) {
                const std::vector<float>& tlwh = track.tlwh;
                int track_id = track.track_id;
                metaData->track_boxes.emplace_back(tlwh[0], tlwh[1], tlwh[0] + tlwh[2], tlwh[1] + tlwh[3], 1.0f, track_id, track_label_);
            }

            for (auto& output_buffer : output_buffers_)
            {
                // printf("Node %s push data to %s\n", name_.c_str(), output_buffer.first.c_str());
                output_buffer.second->push(metaData);
            }    
        }
        if (!has_data)
        {
            std::unique_lock<std::mutex> lock(mutex_);
            cond_var_->wait_for(lock, std::chrono::milliseconds(100), [this] {
                return !running_;  // 等待时检查退出条件
            });
        }
    }
};

}   // namespace Node